@@ -1,4 +1,4 @@
-# Copyright (C) 2003-2006 Robey Pointer <robey@lag.net>
+# Copyright (C) 2003-2007  Robey Pointer <robey@lag.net>
 #
 # This file is part of paramiko.
 #
@@ -149,7 +149,7 @@ class Transport (threading.Thread):
     """
 
     _PROTO_ID = '2.0'
-    _CLIENT_ID = 'paramiko_1.6.4'
+    _CLIENT_ID = 'paramiko_1.7.2'
 
     _preferred_ciphers = ( 'aes128-cbc', 'blowfish-cbc', 'aes256-cbc', '3des-cbc' )
     _preferred_macs = ( 'hmac-sha1', 'hmac-md5', 'hmac-sha1-96', 'hmac-md5-96' )
@@ -274,9 +274,11 @@ class Transport (threading.Thread):
         self.channels = weakref.WeakValueDictionary()   # (id -> Channel)
         self.channel_events = { }       # (id -> Event)
         self.channels_seen = { }        # (id -> True)
-        self.channel_counter = 1
+        self._channel_counter = 1
         self.window_size = 65536
         self.max_packet_size = 34816
+        self._x11_handler = None
+        self._tcp_handler = None
 
         self.saved_exception = None
         self.clear_to_send = threading.Event()
@@ -547,9 +549,9 @@ class Transport (threading.Thread):
             return
         self.active = False
         self.packetizer.close()
+        self.join()
         for chan in self.channels.values():
             chan._unlink()
-        self.join()
 
     def get_remote_server_key(self):
         """
@@ -592,6 +594,35 @@ class Transport (threading.Thread):
         """
         return self.open_channel('session')
 
+    def open_x11_channel(self, src_addr=None):
+        """
+        Request a new channel to the client, of type C{"x11"}.  This
+        is just an alias for C{open_channel('x11', src_addr=src_addr)}.
+
+        @param src_addr: the source address of the x11 server (port is the
+            x11 port, ie. 6010)
+        @type src_addr: (str, int)
+        @return: a new L{Channel}
+        @rtype: L{Channel}
+        
+        @raise SSHException: if the request is rejected or the session ends
+            prematurely
+        """
+        return self.open_channel('x11', src_addr=src_addr)
+    
+    def open_forwarded_tcpip_channel(self, (src_addr, src_port), (dest_addr, dest_port)):
+        """
+        Request a new channel back to the client, of type C{"forwarded-tcpip"}.
+        This is used after a client has requested port forwarding, for sending
+        incoming connections back to the client.
+        
+        @param src_addr: originator's address
+        @param src_port: originator's port
+        @param dest_addr: local (server) connected address
+        @param dest_port: local (server) connected port
+        """
+        return self.open_channel('forwarded-tcpip', (dest_addr, dest_port), (src_addr, src_port))
+    
     def open_channel(self, kind, dest_addr=None, src_addr=None):
         """
         Request a new channel to the server.  L{Channel}s are socket-like
@@ -600,14 +631,14 @@ class Transport (threading.Thread):
         L{connect} or L{start_client}) and authenticating.
 
         @param kind: the kind of channel requested (usually C{"session"},
-            C{"forwarded-tcpip"} or C{"direct-tcpip"})
+            C{"forwarded-tcpip"}, C{"direct-tcpip"}, or C{"x11"})
         @type kind: str
         @param dest_addr: the destination address of this port forwarding,
             if C{kind} is C{"forwarded-tcpip"} or C{"direct-tcpip"} (ignored
             for other channel types)
         @type dest_addr: (str, int)
         @param src_addr: the source address of this port forwarding, if
-            C{kind} is C{"forwarded-tcpip"} or C{"direct-tcpip"}
+            C{kind} is C{"forwarded-tcpip"}, C{"direct-tcpip"}, or C{"x11"}
         @type src_addr: (str, int)
         @return: a new L{Channel} on success
         @rtype: L{Channel}
@@ -621,11 +652,7 @@ class Transport (threading.Thread):
             return None
         self.lock.acquire()
         try:
-            chanid = self.channel_counter
-            while chanid in self.channels:
-                self.channel_counter = (self.channel_counter + 1) & 0xffffff
-                chanid = self.channel_counter
-            self.channel_counter = (self.channel_counter + 1) & 0xffffff
+            chanid = self._next_channel()
             m = Message()
             m.add_byte(chr(MSG_CHANNEL_OPEN))
             m.add_string(kind)
@@ -637,6 +664,9 @@ class Transport (threading.Thread):
                 m.add_int(dest_addr[1])
                 m.add_string(src_addr[0])
                 m.add_int(src_addr[1])
+            elif kind == 'x11':
+                m.add_string(src_addr[0])
+                m.add_int(src_addr[1])
             self.channels[chanid] = chan = Channel(chanid)
             self.channel_events[chanid] = event = threading.Event()
             self.channels_seen[chanid] = True
@@ -665,6 +695,67 @@ class Transport (threading.Thread):
             e = SSHException('Unable to open channel.')
         raise e
 
+    def request_port_forward(self, address, port, handler=None):
+        """
+        Ask the server to forward TCP connections from a listening port on
+        the server, across this SSH session.
+        
+        If a handler is given, that handler is called from a different thread
+        whenever a forwarded connection arrives.  The handler parameters are::
+        
+            handler(channel, (origin_addr, origin_port), (server_addr, server_port))
+            
+        where C{server_addr} and C{server_port} are the address and port that
+        the server was listening on.
+        
+        If no handler is set, the default behavior is to send new incoming
+        forwarded connections into the accept queue, to be picked up via
+        L{accept}.
+        
+        @param address: the address to bind when forwarding
+        @type address: str
+        @param port: the port to forward, or 0 to ask the server to allocate
+            any port
+        @type port: int
+        @param handler: optional handler for incoming forwarded connections
+        @type handler: function(Channel, (str, int), (str, int))
+        @return: the port # allocated by the server
+        @rtype: int
+        
+        @raise SSHException: if the server refused the TCP forward request
+        """
+        if not self.active:
+            raise SSHException('SSH session not active')
+        address = str(address)
+        port = int(port)
+        response = self.global_request('tcpip-forward', (address, port), wait=True)
+        if response is None:
+            raise SSHException('TCP forwarding request denied')
+        if port == 0:
+            port = response.get_int()
+        if handler is None:
+            def default_handler(channel, (src_addr, src_port), (dest_addr, dest_port)):
+                self._queue_incoming_channel(channel)
+            handler = default_handler
+        self._tcp_handler = handler
+        return port
+
+    def cancel_port_forward(self, address, port):
+        """
+        Ask the server to cancel a previous port-forwarding request.  No more
+        connections to the given address & port will be forwarded across this
+        ssh connection.
+        
+        @param address: the address to stop forwarding
+        @type address: str
+        @param port: the port to stop forwarding
+        @type port: int
+        """
+        if not self.active:
+            return
+        self._tcp_handler = None
+        self.global_request('cancel-tcpip-forward', (address, port), wait=True)
+        
     def open_sftp_client(self):
         """
         Create an SFTP client channel from an open transport.  On success,
@@ -1193,8 +1284,7 @@ class Transport (threading.Thread):
         """
         Turn on/off compression.  This will only have an affect before starting
         the transport (ie before calling L{connect}, etc).  By default,
-        compression is off since it negatively affects interactive sessions
-        and is not fully tested.
+        compression is off since it negatively affects interactive sessions.
         
         @param compress: C{True} to ask the remote client/server to compress
             traffic; C{False} to refuse compression
@@ -1230,17 +1320,26 @@ class Transport (threading.Thread):
     ###  internals...
 
     
-    def _log(self, level, msg):
+    def _log(self, level, msg, *args):
         if issubclass(type(msg), list):
             for m in msg:
                 self.logger.log(level, m)
         else:
-            self.logger.log(level, msg)
+            self.logger.log(level, msg, *args)
 
     def _get_modulus_pack(self):
         "used by KexGex to find primes for group exchange"
         return self._modulus_pack
 
+    def _next_channel(self):
+        "you are holding the lock"
+        chanid = self._channel_counter
+        while chanid in self.channels:
+            self._channel_counter = (self._channel_counter + 1) & 0xffffff
+            chanid = self._channel_counter
+        self._channel_counter = (self._channel_counter + 1) & 0xffffff
+        return chanid
+
     def _unlink_channel(self, chanid):
         "used by a Channel to remove itself from the active channel list"
         try:
@@ -1314,6 +1413,24 @@ class Transport (threading.Thread):
             raise SSHException('Unknown client cipher ' + name)
         return self._cipher_info[name]['class'].new(key, self._cipher_info[name]['mode'], iv)
 
+    def _set_x11_handler(self, handler):
+        # only called if a channel has turned on x11 forwarding
+        if handler is None:
+            # by default, use the same mechanism as accept()
+            def default_handler(channel, (src_addr, src_port)):
+                self._queue_incoming_channel(channel)
+            self._x11_handler = default_handler
+        else:
+            self._x11_handler = handler
+
+    def _queue_incoming_channel(self, channel):
+        self.lock.acquire()
+        try:
+            self.server_accepts.append(channel)
+            self.server_accept_cv.notify()
+        finally:
+            self.lock.release()
+    
     def run(self):
         # (use the exposed "run" method, because if we specify a thread target
         # of a private method, threading.Thread will keep a reference to it
@@ -1710,8 +1827,19 @@ class Transport (threading.Thread):
         self._log(DEBUG, 'Received global request "%s"' % kind)
         want_reply = m.get_boolean()
         if not self.server_mode:
-            self._log(DEBUG, 'Rejecting "%s" channel request from server.' % kind)
+            self._log(DEBUG, 'Rejecting "%s" global request from server.' % kind)
             ok = False
+        elif kind == 'tcpip-forward':
+            address = m.get_string()
+            port = m.get_int()
+            ok = self.server_object.check_port_forward_request(address, port)
+            if ok != False:
+                ok = (ok,)
+        elif kind == 'cancel-tcpip-forward':
+            address = m.get_string()
+            port = m.get_int()
+            self.server_object.cancel_port_forward_request(address, port)
+            ok = True
         else:
             ok = self.server_object.check_global_request(kind, m)
         extra = ()
@@ -1784,21 +1912,47 @@ class Transport (threading.Thread):
         initial_window_size = m.get_int()
         max_packet_size = m.get_int()
         reject = False
-        if not self.server_mode:
+        if (kind == 'x11') and (self._x11_handler is not None):
+            origin_addr = m.get_string()
+            origin_port = m.get_int()
+            self._log(DEBUG, 'Incoming x11 connection from %s:%d' % (origin_addr, origin_port))
+            self.lock.acquire()
+            try:
+                my_chanid = self._next_channel()
+            finally:
+                self.lock.release()
+        elif (kind == 'forwarded-tcpip') and (self._tcp_handler is not None):
+            server_addr = m.get_string()
+            server_port = m.get_int()
+            origin_addr = m.get_string()
+            origin_port = m.get_int()
+            self._log(DEBUG, 'Incoming tcp forwarded connection from %s:%d' % (origin_addr, origin_port))
+            self.lock.acquire()
+            try:
+                my_chanid = self._next_channel()
+            finally:
+                self.lock.release()
+        elif not self.server_mode:
             self._log(DEBUG, 'Rejecting "%s" channel request from server.' % kind)
             reject = True
             reason = OPEN_FAILED_ADMINISTRATIVELY_PROHIBITED
         else:
             self.lock.acquire()
             try:
-                my_chanid = self.channel_counter
-                while my_chanid in self.channels:
-                    self.channel_counter = (self.channel_counter + 1) & 0xffffff
-                    my_chanid = self.channel_counter
-                self.channel_counter = (self.channel_counter + 1) & 0xffffff
+                my_chanid = self._next_channel()
             finally:
                 self.lock.release()
-            reason = self.server_object.check_channel_request(kind, my_chanid) 
+            if kind == 'direct-tcpip':
+                # handle direct-tcpip requests comming from the client
+                dest_addr = m.get_string()
+                dest_port = m.get_int()
+                origin_addr = m.get_string()
+                origin_port = m.get_int()
+                reason = self.server_object.check_channel_direct_tcpip_request(
+                                my_chanid, (origin_addr, origin_port), 
+                                           (dest_addr, dest_port))
+            else:
+                reason = self.server_object.check_channel_request(kind, my_chanid)
             if reason != OPEN_SUCCEEDED:
                 self._log(DEBUG, 'Rejecting "%s" channel request from client.' % kind)
                 reject = True
@@ -1811,6 +1965,7 @@ class Transport (threading.Thread):
             msg.add_string('en')
             self._send_message(msg)
             return
+        
         chan = Channel(my_chanid)
         try:
             self.lock.acquire()
@@ -1828,13 +1983,13 @@ class Transport (threading.Thread):
         m.add_int(self.window_size)
         m.add_int(self.max_packet_size)
         self._send_message(m)
-        self._log(INFO, 'Secsh channel %d opened.' % my_chanid)
-        try:
-            self.lock.acquire()
-            self.server_accepts.append(chan)
-            self.server_accept_cv.notify()
-        finally:
-            self.lock.release()
+        self._log(INFO, 'Secsh channel %d (%s) opened.', my_chanid, kind)
+        if kind == 'x11':
+            self._x11_handler(chan, (origin_addr, origin_port))
+        elif kind == 'forwarded-tcpip':
+            self._tcp_handler(chan, (origin_addr, origin_port), (server_addr, server_port))
+        else:
+            self._queue_incoming_channel(chan)
 
     def _parse_debug(self, m):
         always_display = m.get_boolean()
